@@ -1,15 +1,23 @@
-
 #' @name st_as_stars
 #' @param att see \link[raster:factor]{factorValues}; column in the RasterLayer's attribute table
+#' @param ignore_file logical; if \code{TRUE}, ignore the Raster object file name
 #' @export
-st_as_stars.Raster = function(.x, ..., att = 1) {
+st_as_stars.Raster = function(.x, ..., att = 1, ignore_file = FALSE) {
     if (!requireNamespace("raster", quietly = TRUE))
         stop("package raster required, please install it first") # nocov
 
-	if (.x@file@name != "") {
-		r = try(read_stars(.x@file@name, proxy = TRUE), silent = TRUE)
-		if (!inherits(r, "try-error"))
-			return(r)
+	if (!ignore_file) {
+		file = if ("file" %in% slotNames(.x))
+				.x@file@name
+			else if ("filename" %in% slotNames(.x))
+				.x@filename
+			else 
+				""
+		if (file != "") {
+			r = try(read_stars(file, ...), silent = TRUE)
+			if (!inherits(r, "try-error"))
+				return(st_set_crs(r, st_crs(raster::crs(.x))))
+		}
 	}
 
     if (!requireNamespace("sp", quietly = TRUE))
